@@ -191,10 +191,11 @@ class node():
             con_loss = 0
         
         # knowledge sharing
-        reg = 0
         if reg_lam:
             reg = torch.square(torch.norm(torch.cat(tuple([torch.flatten(self.model.state_dict()[k] - reg_model.state_dict()[k])\
                  for k in self.model.state_dict().keys() if 'fc' not in k]),0),2))
+        else:
+            reg, reg_lam = 0, 0
                 
         loss = self.objective(self.model(inputs), labels) + con_loss * mu + reg_lam * reg
         # if self.id == 0:
@@ -261,7 +262,7 @@ class node():
         acc = accuracy_score(label_ts.cpu(), predict_ts.cpu())
         return acc
 
-    def local_test(self):
+    def local_test(self, model_res = None):
         predict_ts = torch.empty(0).to(self.device)
         label_ts = torch.empty(0).to(self.device)
         with torch.no_grad():
@@ -270,6 +271,10 @@ class node():
                 inputs = inputs.to(self.device)
                 labels = torch.flatten(labels)
                 labels = labels.to(self.device, dtype = torch.long)
+                if model_res:
+                    outputs = model_res(inputs) + self.model(inputs)
+                else:
+                    outputs = self.model(inputs) 
                 outputs = self.model(inputs)
                 # print(outputs.data.dtype)
                 _, predicted = torch.max(outputs.data, 1)
@@ -281,7 +286,7 @@ class node():
         # print('Accuracy, Macro F1, Micro F1 of Device %d on the %d test cases: %.2f %%, %.2f, %.2f' % (self.id, len(label_ts), acc*100, macro_f1, micro_f1))
         print('Accuracy, Macro F1 of Device %d on the %d test cases: %.2f %%, %.2f %%' % (self.id, len(label_ts), acc*100, macro_f1*100))
         self.test_metrics.append([acc, macro_f1])
-        # torch.cuda.empty_cache()
+
 
     def local_ensemble_test(self, model_list, voting = 'soft'):
         predict_ts = torch.empty(0).to(self.device)
